#include "tunnels.h"
#include <iostream>
#include <algorithm>
#include <vector>
#include <cassert>

using namespace std;

const int MAXN = 5000;
const int MAXM = 5000;
const int INF = 1000000000;

int n, m, k;
bool blocked[MAXN][MAXM], reach[MAXN+1][MAXM];
int dp[MAXN+1][MAXM];

struct BigBinNumba
{
	vector<int> v;
	bool add(int bit)
	{
		if (bit < 0) return true;
		while (v.size() <= bit) {
			v.push_back(0);
		}
		while (v[bit]) {
			v[bit] = 0;
			bit++;
			if (v.size() <= bit) {
				v.push_back(0);
			}
		}
		v[bit] = 1;
		return true;
	}
	bool subtract(int bit)
	{
		if (bit < 0) return true;
		if (bit >= v.size()) {
			return false;
		}
		while (!v[bit]) {
			v[bit] = 1;
			bit++;
		}
		v[bit] = 0;
		while (!v.empty() && !v.back()) {
			v.pop_back();
		}
		return true;
	}
};
BigBinNumba bs;

int getBestPoz(int i, int l, int r, BigBinNumba &bs)
{
	int best = l;
	while (bs.subtract(dp[i+1][best] + 1)) {
		best++;
	}
	if (best == l) {
		bs.v.clear();
		return l;
	}
	if (best == r) {
		bs.v.clear();
		return r-1;
	}
	if (dp[i+1][best] >= bs.v.size()) best--;
	bs.v.clear();
	return best;
}

int getFunny(int i, int l, int r, BigBinNumba &bs, int legit)
{
	if (!legit) {
		return 0;
	}
	if (legit == 1) {
		bs.v.clear();
		for (int j = l; j <= r; j++) {
			if (reach[i+1][j]) {
				return dp[i+1][j];
			}
		}
	}
	int best = getBestPoz(i, l, r, bs);
	legit = 0;
	for (int j = l; j <= best; j++) {
		bs.add(dp[i+1][j]);
		legit += reach[i+1][j];
	}
	int res = getFunny(i, l, best, bs, legit);
	legit = 0;
	for (int j = best+1; j <= r; j++) {
		bs.add(dp[i+1][j]);
		legit += reach[i+1][j];
	}
	res = max(res, getFunny(i, best+1, r, bs, legit));
	return res + 1;
}

void calcDp()
{
	for (int j = 0; j < m; j++) {
		dp[n][j] = 0;
		reach[n][j] = true;
	}
	for (int i = n-1; i >= 0; i--) {
		for (int l = 0; l < m; l++) {
			if (blocked[i][l]) {
				dp[i][l] = -2;
				reach[i][l] = false;
				continue;
			}
			int r = l;
			bs.v.clear();
			bs.add(dp[i+1][r]);
			int legit = reach[i+1][r];
			while (r < m-1 && !blocked[i][r+1]) {
				r++;
				bs.add(dp[i+1][r]);
				legit += reach[i+1][r];
			}
			if (!legit) {
				for (int j = l; j <= r; j++) {
					dp[i][j] = -2;
					reach[i][j] = false;
				}
			}
			else {
				int res = getFunny(i, l, r, bs, legit);
				for (int j = l; j <= r; j++) {
					dp[i][j] = res;
					reach[i][j] = true;
				}
			}
			l = r;
		}
	}
}

void solve(int N, int M, int K, const vector<vector<bool>> &Blocked)
{
	n = N;
	m = M;
	k = K;
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < m; j++) {
			blocked[i][j] = Blocked[i][j];
		}
	}
	calcDp();
	for (int i = 0; i < n; i++) {
		int l = k, r = k;
		int legit = reach[i+1][k];
		while (l > 0 && !blocked[i][l-1]) {
			l--;
			legit += reach[i+1][l];
		}
		while (r < m-1 && !blocked[i][r+1]) {
			r++;
			legit += reach[i+1][r];
		}
		while (legit > 1) {
			bs.v.clear();
			for (int j = l; j <= r; j++) {
				bs.add(dp[i+1][j]);
			}
			int best = getBestPoz(i, l, r, bs);
			if (best < k) {
				bool res = investigate(best);
				if (res) {
					r = best;
				}
				else {
					l = best+1;
				}
			}
			else {
				bool res = investigate(best+1);
				if (res) {
					l = best+1;
				}
				else {
					r = best;
				}
			}
			legit = 0;
			for (int j = l; j <= r; j++) {
				legit += reach[i+1][j];
			}
		}
		for (int j = l; j <= r; j++) {
			if (reach[i+1][j]) {
				k = j;
				break;
			}
		}
		goDeeper(k);
	}
}
