#include <iostream>
#include <algorithm>
#include <vector>

using namespace std;

#define sz(X) ((int)(X).size())

const int MAXN = 300000;
const int MAXQ = 300000;
const int BULAN = 801;

struct Interval
{
	int l, r;
	Interval(int l, int r) : l(l), r(r) {}
	Interval() : l(), r() {}
	Interval getIntersect(const Interval &other) const
	{
		return Interval(max(l, other.l), min(r, other.r));
	}
	int length() const
	{
		return max(0, r-l+1);
	}
	bool empty() const
	{
		return l > r;
	}
};

int n, q, a[MAXN+2], lastap[MAXN+2];
vector<int> allap[MAXN+2];
long long ans;
vector<pair<int, int>> marses[MAXN+2];

template<int MAXSZ>
struct MaxQueue
{
	pair<int, int> v[MAXSZ+2];
	int n = 0;
	void relabel(int poz)
	{
		while (poz > 1 && v[poz].second > v[poz-1].second) {
			swap(v[poz], v[poz-1]);
			poz--;
		}
	}
	void push(int key, int val)
	{
		for (int i = 1; i <= n; i++) {
			if (v[i].first == key) {
				v[i].second = val;
				relabel(i);
				return;
			}
		}
		v[++n] = {key, val};
		relabel(n);
		while (n > MAXSZ) {
			n--;
		}
	}
	int operator [](int poz)
	{
		return v[poz].first;
	}
};
MaxQueue<BULAN> mq;

template<int MAXSZ>
struct Aint
{
	struct Data
	{
		int val;
		long long xval;
		int st, dr;
		int t;
	};
	Data v[MAXSZ];
	int root = 0, nrnodes = 0;
	int n;
	
	void pull(Data &node, const Data &l, const Data &r)
	{
		node.val = l.val + r.val;
		node.xval = l.xval + r.xval;
	}
	
	void build(int &node, int l, int r)
	{
		if (l < r) {
			int mij = (l+r) / 2;
			build(v[node].st, l, mij);
			build(v[node].dr, mij+1, r);
		}
	}
	void init(int n)
	{
		this->n = n;
		build(root, 1, n);
	}
	
	void update(int &node, int l, int r, int qx, int qy, int qval)
	{
		if (v[node].t != qx) {
			v[++nrnodes] = v[node];
			node = nrnodes;
			v[node].t = qx;
		}
		if (l == r) {
			v[node].val += qval;
			v[node].xval += qx * qval;
			return;
		}
		int mij = (l+r) / 2;
		if (qy <= mij) {
			update(v[node].st, l, mij, qx, qy, qval);
		}
		else {
			update(v[node].dr, mij+1, r, qx, qy, qval);
		}
		pull(v[node], v[v[node].st], v[v[node].dr]);
	}
	Data query(int node, int l, int r, int ql, int qr)
	{
		if (ql <= l && r <= qr) {
			return v[node];;
		}
		int mij = (l+r) / 2;
		if (mij+1 > qr) return query(v[node].st, l, mij, ql, qr);
		if (mij < ql) return query(v[node].dr, mij+1, r, ql, qr);
		Data aux;
		pull(aux, query(v[node].st, l, mij, ql, qr), query(v[node].dr, mij+1, r, ql, qr));
		return aux;
	}
};
Aint<30000000> aint;

int coolroots[MAXN+2];

Interval getGoodInterval(int val)
{
	if (sz(allap[val]) < val) return Interval(1, 0);
	if (sz(allap[val]) == val) return Interval(1, allap[val][0]);
	return Interval(allap[val][allap[val].size()-val-1] + 1, allap[val][allap[val].size()-val]);
}

void init(int N, int Q, const vector<int> &A)
{
	n = N;
	q = Q;
	for (int i = 1; i <= n; i++) {
		a[i] = A[i-1];
	}
	ans = 0;
	for (int i = 1; i <= n; i++) {
		lastap[a[i]] = i;
		allap[a[i]].push_back(i);
		mq.push(a[i], lastap[a[i]]);
		long long aux = 0;
		Interval good(1, i);
		for (int j = 1; j <= mq.n; j++) {
			good = good.getIntersect(getGoodInterval(mq.v[j].first));
			if (j == mq.n) {
				if (!good.empty()) {
					aux += good.length();
					marses[good.r].push_back({i, +1});
					marses[good.l-1].push_back({i, -1});
				}
			}
			else {
				Interval good2 = good.getIntersect(Interval(lastap[mq.v[j+1].first] + 1, i));
				if (!good2.empty()) {
					aux += good2.length();
					marses[good2.r].push_back({i, +1});
					marses[good2.l-1].push_back({i, -1});
				}
			}
		}
		ans += aux;
	}
	aint.init(n);
	for (int x = n; x >= 1; x--) {
		for (auto &p : marses[x]) {
			aint.update(aint.root, 1, n, x, p.first, p.second);
		}
		coolroots[x] = aint.root;
	}
}

long long query(int l, int r)
{
	l++;
	r++;
	if (l == 1 && r == n) {
		return ans;
	}
	auto res = aint.query(coolroots[l], 1, n, 1, r);
	return res.xval - 1LL * res.val * (l-1);
}
