#include "tunnels.h"
#include <iostream>
#include <vector>
#include <tuple>
#include <functional>
#include <cassert>
#include <map>
#include <set>
#define all(x) (x).begin(),(x).end()

using namespace std;

using ll = long long;
// #define int ll
#define sz(x) ((int)(x).size())

using pii = pair<int, int>;
using tii = tuple<int, int, int>;


const int nmax = 5e3 + 5;
const int inf = 1e8 + 5;

vector<int> cost;
vector<pii> limitsegm;
vector<int> solved, seen;
vector<vector<int>> ordersons;

int atr[nmax][nmax];

pii intersect(pii a, pii b) {
   return pii{max(a.first, b.first), min(a.second, b.second)};
}
int intersectSize(pii a, pii b) {
   auto c = intersect(a, b);
   if(c.first > c.second) return 0;
   return c.second - c.first + 1;
}

pair<vector<int>, int> solveunder(vector<tii> a, int L) {
   if(sz(a) == 1) {
      if(get<1>(a[0]) + get<2>(a[0]) - 1 <= L) {
	 return make_pair(vector<int>{get<0>(a[0])}, L);
      }
      return make_pair(vector<int>{}, -1);
   }
   assert(false);
}

void solve(int C, vector<tii> descendants) {
   int lim = -1;
   for(int i = 24; i >= 0; i--) {
      auto [order, pl] = solveunder(descendants, lim + (1 << i));
      if(!sz(order)) lim += (1 << i); 
   }
   tie(ordersons[C], cost[C]) = solveunder(descendants, lim + 1);
}

void solve(int n, int m, int k, const std::vector<std::vector<bool>>& blocked) {
   int flag = -1;
   for(int i = 0; i < n; i++) {
      flag++;
      cost.emplace_back(0);
      limitsegm.emplace_back(n, 0);
      for(int j = 0; j < n; j++) {
	 if(atr[i][j]) {
	    cost.emplace_back(0);
	    limitsegm.emplace_back(n, 0);
	    flag++;
	 }
	 else {
	    atr[i][j] = flag;
	    limitsegm.back().first = min(limitsegm.back().first, j);
	    limitsegm.back().second = max(limitsegm.back().second, j);
	 }
      }
   }
   solved.assign(flag + 1, 0);
   seen.assign(flag + 1, 0);
   ordersons.assign(flag + 1, vector<int>{});
   
   int occflag = 0;
   // cerr << flag << '\n';
   // cerr << "Yes\n";
   
   
   for(int i = n - 1; i >= 0; i--) {
      for(int j = 0; j < m; j++) {
	 if(blocked[i][j]) continue;
	 // cerr << atr[i][j] << ' ' << sz(solved) << '\n';
	 if(solved[atr[i][j]]) continue;
	 
	 
	 occflag++;
	 vector<tii> descendants;
	 int C = atr[i][j];
	 
	 solved[C] = 1;
	 if(i == n - 1) {
	    cost[C] = limitsegm[C].second - limitsegm[C].first;
	    continue;
	 }
	 
	 
	 for(int a = limitsegm[C].first; a <= limitsegm[C].second; a++) {
	    if(blocked[i + 1][a]) continue;
	    if(seen[atr[i + 1][a]] == occflag) continue;
	    seen[atr[i + 1][a]] = occflag;
	    descendants.emplace_back(atr[i + 1][a], intersectSize(limitsegm[atr[i + 1][a]], limitsegm[C]), cost[atr[i + 1][a]]);
	 }
	 if(sz(descendants) == 0) {
	    cost[C] = -inf;
	 }
	 else {
	    solve(C, descendants);
	    if(cost[C] < 0) cost[C] = -inf;
	 }
      }
   }
   
   // cerr << "No\n";
   
   int line = 0, col = k;
   while(line < n) {
      if(line == n - 1) {
	 for(int i = limitsegm[atr[line][col]].first; i <= limitsegm[atr[line][col]].second; i++) {
	    if(i == limitsegm[atr[line][col]].second) {
	       goDeeper(i);
	    }
	    else {
	       if(investigate(i)) { goDeeper(i); break; }
	    }
	 }
	 break;
      }
      
      auto &v = ordersons[atr[line][col]];
      assert(sz(v));
      for(int i = 0; i < sz(v); i++) {
	 auto [l, r] = intersect(limitsegm[v[i]], limitsegm[atr[line][col]]);
	 int good = -1;
	 for(int j = l; j <= r; j++) {
	    if(j == r && i == sz(v) - 1) { good = r; break; }
	    if(investigate(j)) { good = j; break; }
	 }
	 if(good != -1) {
	    goDeeper(good);
	    line++;
	    col = good;
	    break;
	 }
      }
   }
   
   
}
