#include "tunnels.h"
#include <vector>
#include <stdio.h>

using namespace std;

struct seg {
    int l, r;
};

const int MAX_N = 100;
const int INF = 1e9;
int N;
int leftt[MAX_N][MAX_N], rightt[MAX_N][MAX_N];
int dp[MAX_N][MAX_N], opt[MAX_N][MAX_N][MAX_N], costTotal[MAX_N][MAX_N];

void computeDp( int lin, seg cols ) {
    for ( int i = cols.l; i <= cols.r; i++ )
        dp[i][i] = costTotal[lin + 1][i]; 

    for ( int len = 1; len <= cols.r - cols.l; len++ ) {
        for ( int l = cols.l; l <= cols.r - len; l++ ) {
            int r = l + len;
            dp[l][r] = INF;
            for ( int k = l; k <= r - 1; k++ ) {
                if ( 1 + max( dp[l][k], dp[k + 1][r] ) <= dp[l][r] ) {
                    dp[l][r] = 1 + max( dp[l][k], dp[k + 1][r] );
                    opt[lin][l][r] = k;
                }
            }
            //printf( "%d %d %d: %d %d\n", lin, l, r, dp[l][r], opt[lin][l][r] );
        }
    }

    for ( int i = cols.l; i <= cols.r; i++ )
        costTotal[lin][i] = dp[cols.l][cols.r];
}

void getToTreasure( int lin, int k ) {
    if ( lin == N )
        return;

    int l = leftt[lin][k], r = rightt[lin][k];
    while ( l < r ) {
        int p = opt[lin][l][r];
        if ( p >= k )
            p++;
        //printf( "%d %d %d %d %d\n", lin, l, r, p, k );
        bool a = investigate( p );
        if ( a ) {
            if ( p < k )
                r = p;
            else
                l = p;
        } else {
            if ( p < k )
                l = p + 1;
            else
                r = p - 1;
        }
    }

    goDeeper( l );
    getToTreasure( lin + 1, l );
}

void solve( int n, int m, int k, const vector<vector<bool>> &blocked ) {
    N = n;
    for ( int l = n - 1; l >= 0; l-- ) {
        int ll = 0, rr = -1;
        vector<seg> segs;
        for ( int c = 0; c < m; c++ ) {
            if ( blocked[l][c] ) {
                if ( rr >= ll )
                    segs.push_back( { ll, rr } );
                ll = c + 1;
                rr = c;
            } else
                rr = c;
        }   
        if ( rr >= ll )
            segs.push_back( { ll, rr } );
        for ( seg s: segs ) {
            for ( int i = s.l; i <= s.r; i++ )
                leftt[l][i] = s.l, rightt[l][i] = s.r;
            computeDp( l, s );
        }
    }
    getToTreasure( 0, k );
}
