#include "tunnels.h"
#include <vector>
#include <stdio.h>

using namespace std;

struct seg {
    int l, r;
};

const int MAX_N = 5000;
const int MAX_M = 100;
const int INF = 1e9;
int N;
int leftt[MAX_N][MAX_N], rightt[MAX_N][MAX_N];
int dp[MAX_N][MAX_N], opt[MAX_N][MAX_M][MAX_M], costTotal[MAX_N + 1][MAX_N];
vector<int> rel[MAX_N];

void computeDp( int lin, seg cols ) {
    for ( int i = cols.l; i <= cols.r; i++ )
        dp[i][i] = costTotal[lin + 1][i]; 

    for ( int len = 1; len <= cols.r - cols.l; len++ ) {
        for ( int l = cols.l; l <= cols.r - len; l++ ) {
            int r = l + len;
            dp[l][r] = INF;
            for ( int k = l; k <= r - 1; k++ ) {
                if ( 1 + max( dp[l][k], dp[k + 1][r] ) <= dp[l][r] ) {
                    dp[l][r] = 1 + max( dp[l][k], dp[k + 1][r] );
                    opt[lin][l][r] = k;
                }
            }
            //printf( "%d %d %d: %d %d\n", lin, l, r, dp[l][r], opt[lin][l][r] );
        }
    }

    for ( int i = cols.l; i <= cols.r; i++ )
        costTotal[lin][i] = dp[cols.l][cols.r];
}

void getToTreasure( int lin, int k ) {
    if ( lin == N )
        return;

    int l = -1, r = -1;
    for ( int i = 0; i < (int)rel[lin].size(); i++ ) {
        int c = rel[lin][i];
        if ( c >= leftt[lin][k] && c <= rightt[lin][k] ) {
            if ( l == -1 )
                l = i;
            r = i;
        }
    }
    while ( l < r ) {
        int p = opt[lin][l][r];
        if ( rel[lin][p] >= k )
            p++;
        //printf( "%d %d %d %d %d\n", lin, l, r, p, k );
        bool a = investigate( p );
        if ( a ) {
            if ( p < k )
                r = p;
            else
                l = p;
        } else {
            if ( p < k )
                l = p + 1;
            else
                r = p - 1;
        }
    }

    goDeeper( rel[lin][l] );
    getToTreasure( lin + 1, rel[lin][l] );
}

void solve( int n, int m, int k, const vector<vector<bool>> &blocked ) {
    N = n;
    for ( int l = 0; l <= n; l++ ) {
        for ( int c = 0; c < m; c++ )
            costTotal[l][c] = -INF;
    }
    for ( int l = 0; l < n; l++ ) {
        rightt[l][m - 1] = (blocked[l][m - 1] ? m - 2 : m - 1);
        for ( int c = m - 2; c >= 0; c-- )
            rightt[l][c] = (blocked[l][c] ? c - 1 : rightt[l][c + 1]);
        leftt[l][0] = (blocked[l][0] ? 1 : 0);
        for ( int c = 1; c < m; c++ )
            leftt[l][c] = (blocked[l][c] ? c + 1 : leftt[l][c - 1]);

    }
    for ( int l = n - 1; l >= 0; l-- ) {
        int ll = 0, rr = -1;
        vector<seg> segs;
        for ( int c = 0; c < m; c++ ) {
            if ( l == n - 1 || !blocked[l + 1][c] )
                rel[l].push_back( c );// printf( "%d %d\n", l, c );;
        }
        for ( int i = 0; i < (int)rel[l].size(); i++ ) {
            int c = rel[l][i];
            if ( blocked[l][c] ) {
                if ( rr >= ll )
                    segs.push_back( { ll, rr } );
                ll = i + 1;
                rr = i;
            } else
                rr = i;
        }   
        if ( rr >= ll )
            segs.push_back( { ll, rr } );
        for ( seg s: segs ) 
            computeDp( l, s );
    }
    getToTreasure( 0, k );
}
