#include "tunnels.h"
#include <vector>
#include <queue>
#include <stdio.h>

using namespace std;

struct seg {
    int l, r;
};

struct nodeType {
    int lin, l, r, cost, opt;
};

struct pqelem {
    int c, u, v;
    bool operator < ( const pqelem &x ) const {
        return c > x.c;
    }
};

const int MAX_N = 5005;
const int MAX_NODES = 2e6;
const int INF = 1e9;
int N;
int leftt[MAX_N][MAX_N], rightt[MAX_N][MAX_N];
int leftSide[MAX_N], rightSide[MAX_N], node[MAX_N];
int costTotal[MAX_N + 1][MAX_N];
vector<nodeType> nodes;
int leftSon[MAX_NODES], rightSon[MAX_NODES];
bool used[MAX_NODES];
vector<int> rel[MAX_N];

pair<int, int> getInterval( int lin, int k ) {
    int l = -1, r = -1;
    for ( int i = 0; i < (int)rel[lin].size(); i++ ) {
        int c = rel[lin][i];
        if ( c >= leftt[lin][k] && c <= rightt[lin][k] ) {
            if ( l == -1 )
                l = i;
            r = i;
        }
    }
    return { l, r };
}

void buildTree( int lin, seg cols ) {
    //printf( "START %d %d %d\n", lin, cols.l, cols.r );
    for ( int i = cols.l; i <= cols.r; i++ ) {
        leftSide[i] = i;
        rightSide[i] = i;
        node[i] = nodes.size();
        nodes.push_back( { lin, i, i, costTotal[lin + 1][rel[lin][i]], i } );
    }

    priority_queue<pqelem> pq;
    for ( int i = cols.l; i < cols.r; i++ )
        pq.push( { 1 + max( nodes[node[i]].cost, nodes[node[i + 1]].cost ), node[i], node[i + 1] } );
    
    while ( !pq.empty() ) {
        pqelem p = pq.top();
        pq.pop();

        int u = p.u, v = p.v, c = p.c;
        if ( used[u] || used[v] )
            continue;

        //printf( "merge %d %d\n", u, v );
        used[u] = used[v] = true;
        int l = nodes[u].l, r = nodes[v].r;
        nodes.push_back( { lin, l, r, p.c, nodes[u].r } );
        node[l] = node[r] = nodes.size() - 1;
        leftSon[nodes.size() - 1] = u;
        rightSon[nodes.size() - 1] = v;

        if ( l > cols.l ) {
            int u = node[l - 1], v = node[l];
            pq.push( { 1 + max( nodes[u].cost, nodes[v].cost ), u, v } );
        }
        if ( r < cols.r ) {
            int u = node[l], v = node[r + 1];
            pq.push( { 1 + max( nodes[u].cost, nodes[v].cost ), u, v } );
        }
    }

    pair<int, int> pp = { leftt[lin][rel[lin][cols.l]], rightt[lin][rel[lin][cols.l]] };
    for ( int col = pp.first; col <= pp.second; col++ )
        costTotal[lin][col] = nodes.back().cost;
    //printf( "%d %d %d\n", pp.first, pp.second, nodes.back().cost );
    //printf( "FINISH\n" );
}


int getNode( int lin, int l, int r ) {
    for ( int i = 0; i < nodes.size(); i++ ) {
        if ( nodes[i].lin == lin && nodes[i].l == l && nodes[i].r == r )
            return i;
    }
    return 0;
}

void solve( int n, int m, int k, const vector<vector<bool>> &blocked ) {
    N = n;
    for ( int l = 0; l < n; l++ ) {
        for ( int c = 0; c < m; c++ )
            costTotal[l][c] = -INF;
    }
    for ( int l = 0; l < n; l++ ) {
        rightt[l][m - 1] = (blocked[l][m - 1] ? m - 2 : m - 1);
        for ( int c = m - 2; c >= 0; c-- )
            rightt[l][c] = (blocked[l][c] ? c - 1 : rightt[l][c + 1]);
        leftt[l][0] = (blocked[l][0] ? 1 : 0);
        for ( int c = 1; c < m; c++ )
            leftt[l][c] = (blocked[l][c] ? c + 1 : leftt[l][c - 1]);
    }
    for ( int l = n - 1; l >= 0; l-- ) {
        int ll = -1, rr = -1;
        vector<seg> segs;
        for ( int c = 0; c < m; c++ ) {
            if ( !blocked[l][c] && costTotal[l + 1][c] >= 0 ) {
                rel[l].push_back( c );//, printf( "%d %d\n", l, c );;
                if ( ll == -1 )
                    ll = rel[l].size() - 1;
                rr = rel[l].size() - 1;
            }
            if ( blocked[l][c] ) {
                if ( ll != -1 )
                    segs.push_back( { ll, rr } );
                ll = rr = -1;
            }
        }
        if ( ll != -1 )
            segs.push_back( { ll, rr } );

        for ( seg s: segs )  {
//            printf( "%d %d\n", s.l, s.r );
            buildTree( l, s );
        }
    }
    //return;
    for ( int lin = 0; lin < n; lin++ ) {
        auto pp = getInterval( lin, k );
        int l = pp.first, r = pp.second;
        int u = getNode( lin, l, r );
        //printf( "SUNT LA %d %d %d\n", l, r, u );

        while ( l < r ) {
            int p = nodes[u].opt;
            if ( rel[lin][p] >= k )
                p++;

            bool a = investigate( rel[lin][p] );
            if ( a ) {
                if ( rel[lin][p] < k )
                    u = rightSon[u];
                else
                    u = leftSon[u];
            } else {
                if ( rel[lin][p] < k)
                    u = leftSon[u];
                else
                    u = rightSon[u];
            }
            l = nodes[u].l;
            r = nodes[u].r;
        }

        k = rel[lin][l];
        goDeeper( k );
    }
}
