#include "bits/stdc++.h"

using namespace std;

typedef long long ll;

struct ST {
    struct node {
        ll mn, cnt, pl;

        node() : mn(0), cnt(0), pl(0) {}
    };

    ll n;
    vector<node> t;

    ST(ll n) : n(n), t(4 * n) { build(1, 0, n); }

    node merge(node a, node b) {
        node c;
        c.mn = min(a.mn, b.mn);
        c.cnt = (a.mn == c.mn ? a.cnt : 0) + (b.mn == c.mn ? b.cnt : 0);
        return c;
    }

    void update(ll v, ll x) {
        t[v].mn += x;
        t[v].pl += x;
    }

    void push(ll v) {
        update(v * 2, t[v].pl);
        update(v * 2 + 1, t[v].pl);
        t[v].pl = 0;
    }

    void build(ll v, ll l, ll r) {
        if (l + 1 == r) {
            t[v].cnt = 1;
            return;
        }
        ll m = (l + r) / 2;
        push(v);
        build(v * 2, l, m);
        build(v * 2 + 1, m, r);
        t[v] = merge(t[v * 2], t[v * 2 + 1]);
    }

    void update(ll l, ll r, ll x) {
        if (l >= r) return;
        update(1, 0, n, l, r, x);
    }

    void update(ll v, ll l, ll r, ll L, ll R, ll x) {
        if (l >= R || r <= L) return;
        if (l >= L && r <= R) {
            update(v, x);
            return;
        }
        push(v);
        ll m = (l + r) / 2;
        update(v * 2, l, m, L, R, x);
        update(v * 2 + 1, m, r, L, R, x);
        t[v] = merge(t[v * 2], t[v * 2 + 1]);
    }

    vector<ll> get(ll l) {
        vector<ll> rt;
        get(1, 0, n, l, n, rt);
        return rt;
    }

    void get(ll v, ll l, ll r, ll L, ll R, vector<ll> &rt) {
        if (l >= R || r <= L || t[v].mn) return;
        if (l + 1 == r) {
            rt.push_back(l);
            return;
        }
        push(v);
        ll m = (l + r) / 2;
        get(v * 2, l, m, L, R, rt), get(v * 2 + 1, m, r, L, R, rt);
    }
};

struct node {
    node *l, *r;
    ll x;

    node() : x(0), l(0), r(0) {}
};


node *copy(node *v1) {
    node *v = new node();
    v->l = v1->l;
    v->r = v1->r;
    v->x = v1->x;
    return v;
}

struct PST {

    node *V;
    ll n;

    PST(ll n) : n(n), V(new node()) { build(V, 0, n); }

    void build(node *v, ll l, ll r) {
        if (l + 1 == r) return;
        v->l = new node();
        v->r = new node();
        ll m = (l + r) / 2;
        build(v->l, l, m);
        build(v->r, m, r);
    }

    void update(ll p) {
        V = copy(V);
        update(V, 0, n, p);
    }

    void update(node *v, ll l, ll r, ll p) {
        if (l > p || r <= p) return;
        if (l + 1 == r) {
            v->x++;
            return;
        }
        ll m = (l + r) / 2;
        v->l = copy(v->l);
        v->r = copy(v->r);
        update(v->l, l, m, p);
        update(v->r, m, r, p);
        v->x = v->l->x + v->r->x;
    }
};

ll get(node *v, ll l, ll r, ll L, ll R) {
    if (l >= R || r <= L) return 0;
    if (l + 1 == r) {
        return v->x;
    }
    ll m = (l + r) / 2;
    return get(v->l, l, m, L, R) + get(v->r, m, r, L, R);
}

const int N = 3e5 + 5;

ll n, q, a[N];
deque<ll> dq[N];
node *vt[N];

void init(int _n, int _q, const std::vector<int> &_a) {
    n = _n;
    q = _q;
    for (ll i = 0; i < n; i++) a[i] = _a[i];
    for (ll i = 0; i < N; i++) dq[i].push_front(n);

    ST t(n);
    PST pt(n);
    for (ll i = n - 1; i >= 0; i--) {
        if (a[i] >= N) {
            t.update(i, n, 1);
        } else {
            ll l, r;
            l = dq[a[i]][0], r = (dq[a[i]].size() >= a[i] ? dq[a[i]][a[i] - 1] : n);
            t.update(l, r, -1);
            if (dq[a[i]].size() > a[i]) t.update(dq[a[i]][a[i]], n, -1);

            dq[a[i]].push_front(i);

            l = dq[a[i]][0], r = (dq[a[i]].size() >= a[i] ? dq[a[i]][a[i] - 1] : n);
            t.update(l, r, +1);
            if (dq[a[i]].size() > a[i]) t.update(dq[a[i]][a[i]], n, +1);
        }
        auto rt = t.get(i);
        for (auto x: rt) pt.update(x);
        vt[i] = copy(pt.V);
    }
}

ll query(int l, int r) {
    return get(vt[l], 0, n, 0, r + 1);
}