#include "bits_and_tree.h"
#include <bits/stdc++.h>

using namespace std;
using ll = long long;

std::vector<std::pair<int, int>> encode(int n, std::vector<bool> data) {
    {
        mt19937 rnd(228);
        for (int i = 0; i < data.size(); ++i) {
            if (rnd() & 1) {
                data[i] = !data[i];
            }
        }
    }
    vector<pair<int, int>> edges;
    int top = 2;
    for (int i = 1; i <= 5; ++i) {
        edges.push_back({1, top++});
    }
    for (int i = 1; i <= 7; ++i) {
        edges.push_back({0, top++});
    }
    bool xored = false;
    if (data[0] == 1) {
        xored = true;
        for (int i = 0; i < data.size(); ++i) {
            data[i] = !data[i];
        }
        for (int i = 1; i <= 2; ++i) {
            edges.push_back({0, top++});
        }
    }
    // 0 is the start (vertex with the highest degree)
    // 1 is the end (vertex with the highest degree except 0, and if there are several such, the furthest from vertex 0)
    int prv = 0;
    vector<int> path{0};
    // cerr << "encode: ";
    for (int i = 0, j = 0; i < data.size() && top < n; i = j) {
        while (j < data.size() && data[i] == data[j]) {
            j += 1;
        }
        int cnt = j - i;
        // cerr << cnt << " ";
        int x = top++;
        edges.push_back({prv, x});
        prv = x;
        path.push_back(x);
        int rem = n - top;
        if (cnt == 1 || rem <= 1) {
            continue;
        }
        int now = min(rem, cnt);
        int last = x;
        for (int k = 1; k + 2 <= now; ++k) {
            edges.push_back({last, top});
            last = top++;
        }
        edges.push_back({last, top++});
        edges.push_back({last, top++});
        assert(top <= n);
        prv = x;
    }
    // cerr << endl;
    edges.push_back({prv, 1});
    path.push_back(1);
    // cerr << "path: ";
    for (int x : path) {
        // cerr << x << " ";
    }
    // cerr << endl;
    return edges;
}

std::vector<bool> decode(int n, std::vector<std::pair<int, int>> tree) {
    vector<bool> res;
    bool xored = false;
    int start = 0;
    n += 1;
    vector<set<int>> adj(n);
    for (int i = 0; i < tree.size(); ++i) {
        auto [a, b] = tree[i];
        adj[a].insert(b);
        adj[b].insert(a);
    }
    for (int i = 0; i < n; ++i) {
        if (adj[start].size() < adj[i].size()) {
            start = i;
        }
    }
    xored = adj[start].size() > 9;
//    cerr << adj[start].size() << endl;
    auto bfs = [&](int source) -> array<vector<int>, 2> {
        vector<int> dist(n, -1), fa(n, -1);
        dist[source] = 0;
        vector<int> ord;
        ord.push_back(source);
        for (int i = 0; i < ord.size(); ++i) {
            int x = ord[i];
            for (int to : adj[x]) {
                if (dist[to] == -1) {
                    dist[to] = dist[x] + 1;
                    fa[to] = x;
                    ord.push_back(to);
                }
            }
        }
        return {dist, fa};
    };
    auto [dist_start, fa_start] = bfs(start);
    int end = -1;
    for (int i = 0; i < n; ++i) {
        if (i == start) {
            continue;
        }
        if (end == -1 || pair(adj[end].size(), dist_start[end]) < pair(adj[i].size(), dist_start[i])) {
            end = i;
        }
    }
    vector<int> path;
    vector<char> inPath(n);
    {
        int f = fa_start[end];
        inPath[end] = inPath[start] = true;
        while (f != start) {
            path.push_back(f);
            inPath[f] = true;
            f = fa_start[f];
        }
        reverse(path.begin(), path.end());
    }
    for (int x : path) {
        vector<int> nxt;
        for (int to : adj[x]) {
            if (!inPath[to]) {
                nxt.push_back(to);
            }
        }
        adj[x] = set(nxt.begin(), nxt.end());
    }
    int val = 1;
    bool was = false;

    // cerr << "decode: ";
    for (int x : path) {
        val ^= 1;
        auto dist = bfs(x)[0];
        vector<int> cnt(n + 1);
        int sum = 0;
        for (int i = 0; i < n; ++i) {
            if (dist[i] != -1) {
                sum += 1;
                cnt[dist[i]] += 1;
            }
        }
        sum -= 1;
        // cerr << sum << " ";
        if (sum <= 1) {
//            // cerr << "case 1: " << endl;
            if (sum == 1) {
                assert(!was);
                was = true;
            }
            res.push_back(val);
            continue;
        }
        // either sum or sum-1, depending on whether the corrupted vertex is here
        int mx = *max_element(cnt.begin(), cnt.end());
        bool corrupted = false;
        int distinct = 0;
        int ones = 0;
        for (int i = 0; i < cnt.size(); ++i) {
            if (cnt[i] > 0) {
                distinct += 1;
                ones += cnt[i] == 1;
            }
        }
        if (cnt[*max_element(dist.begin(), dist.end())] != 2 || mx > 2 || count(cnt.begin(), cnt.end(), mx) > 2 || ones + 1 != distinct) {
            corrupted = true;
//            // cerr << "case 2" << endl;
            assert(!was);
            was = true;
        }
        sum -= corrupted;
        for (int i = 1; i <= sum; ++i) {
            res.push_back(val);
        }
    }
    // cerr << endl;
    {
        mt19937 rnd(228);
        // cerr << "res: ";
        for (int i = 0; i < res.size(); ++i) {
            if (rnd() & 1) {
                res[i] = !res[i];
            }
            if (xored) {
                res[i] = !res[i];
            }
            // cerr << res[i];
        }
        // cerr << endl;
    }
    return res;
}