#include "bits_and_tree.h"

#include <iostream>
#include <algorithm>
#include <random>
#include <cassert>

using namespace std;

const int K = 1;

std::vector<std::pair<int, int>> encode(int n, std::vector<bool> data) {
    int x = 0;
    for (int i = 0; i < K; ++i) {
        x *= 2;
        if (data[i]) x++;
    }
    ++x;
    vector<pair<int, int>> tr;
    for (int i = 0; i < x * 2 - 1; ++i) {
        tr.push_back({i, i + 1});
    }
    for (int i = x * 2; i < n; ++i) {
        tr.push_back({0, i});
    }
    return tr;
}

std::vector<bool> decode(int n, std::vector<std::pair<int, int>> tree) {
    ++n;
    vector<vector<int>> gr(n);
    for (auto e : tree) {
        gr[e.first].push_back(e.second);
        gr[e.second].push_back(e.first);
    }
    int st = 0;
    {
        vector<int> d(n, n + 5);
        d[st] = 0;
        vector<int> cur;
        cur.push_back(st);
        while (cur.size()) {
            int v = cur.back();
            cur.pop_back();
            for (auto e : gr[v]) {
                if (d[e] < d[v] + 1) continue;
                d[e] = d[v] + 1;
                cur.push_back(e);
            }
        }
        for (int i = 0; i < n; ++i) {
            if (d[i] > d[st]) st = i;
        }
    }
    {
        vector<int> d(n, n + 5);
        d[st] = 0;
        vector<int> cur;
        cur.push_back(st);
        while (cur.size()) {
            int v = cur.back();
            cur.pop_back();
            for (auto e : gr[v]) {
                if (d[e] < d[v] + 1) continue;
                d[e] = d[v] + 1;
                cur.push_back(e);
            }
        }
        for (int i = 0; i < n; ++i) {
            if (d[i] > d[st]) st = i;
        }
        int x = d[st];
        x /= 2;
        --x;
        // cout << d[st] << ' ' << x << '\n';
        vector<bool> ans(n);
        for (int i = K - 1; i >= 0; --i) {
            if (x % 2) ans[i] = 1;
            x /= 2;
        }
        return ans;
    }
    vector<bool> ans(n);
    return ans;
}

#ifdef LOCAL
static void dfs(int u, vector<bool> &vis, const vector<vector<int>> &adj) {
    vis[u] = true;
    
    for (int to: adj[u]) {
        if (vis[to]) {
            continue;
        }

        dfs(to, vis, adj);
    }
}

static bool forms_tree(vector<pair<int, int>> edges, int n) {
    vector<vector<int>> adj(n);
    for (auto [u, v]: edges) {
    
        if (u < 0 || v < 0 || u >= n || v >= n) {
            return false;
        }
        if (u == v) {
            return false;
        }

        adj[u].push_back(v);
        adj[v].push_back(u);
    }

    vector<bool> vis(n, false);

    dfs(0, vis, adj);

    for (int i = 0; i < n; ++i) {
        if (!vis[i]) {
            return false;
        }
    }

    return true;
}

const int RANDOM_SEED = 0;
mt19937 random_generator(RANDOM_SEED);

static vector<pair<int, int>> corrupt(vector<pair<int, int>> edges, int n, int node_to_attach_to) {
    edges.push_back({node_to_attach_to, n});

    vector<int> permutation(n + 1);
    for (int i = 0; i <= n; ++i){
        permutation[i] = i;
    }

    shuffle(permutation.begin(), permutation.end(), random_generator);
    shuffle(edges.begin(), edges.end(), random_generator);

    for (auto &[u, v]: edges) {
        u = permutation[u];
        v = permutation[v];

        if (random_generator() % 2) {
            swap(u, v);
        }
    }

    return edges;
}

static int runTest() {
    int n;
    cout << "n: ";
    cin >> n;

    int data_length;
    cout << "data_length: ";
    cin >> data_length;

    vector<bool> data(data_length);
    cout << "data: ";
    for (int i = 0; i < data_length; ++i) {
        char bit;
        cin >> bit;

        data[i] = bit == '1';
    }

    vector<pair<int, int>> tree_edges = encode(n, data);

    assert(forms_tree(tree_edges, n));

    int min_prefix = 1e9;

    for (int node_to_attach_to = 0; node_to_attach_to < n; ++node_to_attach_to) {
        vector<pair<int, int>> corrupted_edges = corrupt(tree_edges, n, node_to_attach_to);

        vector<bool> decoded_data = decode(n, corrupted_edges);

        int correct_prefix = 0;
        while (correct_prefix < (int) std::min(decoded_data.size(), data.size())
            && decoded_data[correct_prefix] == data[correct_prefix]) {
            ++correct_prefix;
        }
        min_prefix = min(min_prefix, correct_prefix);
    }

    return min_prefix;
}

int main() {
    freopen("in.txt", "r", stdin);
    freopen("out.txt", "w", stdout);
    cout << "t: ";
    int t;
    cin >> t;

    int min_bits_guessed = 1e9;
    for (int i = 0; i < t; i++) {
        int bits = runTest();
        cout << "Guessed " << bits << " bits correctly in test #" << i << endl;

        min_bits_guessed = min(min_bits_guessed, bits);
    }
    cout << endl;
    cout << "Guessed a minimum of " << min_bits_guessed << " correctly" << endl;

    return 0;
}
#endif