#include <bits/stdc++.h>

using namespace std;

struct Segment_tree{
    struct Node{
        long long l, r;
        long long mi;
        long long mod;
        Node() {};
        Node(long long x, long long ind) {
            l = ind;
            r = ind + 1;
            mi = x;
            mod = 0;
        }
        Node(Node le, Node ri) {
            l = le.l;
            r = ri.r;
            mi = min(le.mi, ri.mi);
            mod = 0;
        }
        void add_mod(long long x) {
            mod += x;
        }
        void apply_mod() {
            mi += mod;
            mod = 0;
        }
    };
    vector<Node> tr;
    void build(long long i, long long l, long long r) {
        if (r - l == 1) {
            tr[i] = Node(0, l);
            return;
        }
        long long m = (l + r) / 2;
        build(2 * i + 1, l, m);
        build(2 * i + 2, m, r);
        tr[i] = Node(tr[2 * i + 1], tr[2 * i + 2]);
    }
    void push(long long i) {
        if (tr[i].r - tr[i].l > 1) {
            tr[2 * i + 1].add_mod(tr[i].mod);
            tr[2 * i + 2].add_mod(tr[i].mod);
        }
        tr[i].apply_mod();
    }
    void add(long long l, long long r, long long x, long long i = 0) {
        push(i);
        if (tr[i].l >= r || tr[i].r <= l) {
            return;
        }
        if (l <= tr[i].l && tr[i].r <= r) {
            tr[i].add_mod(x);
            push(i);
            return;
        }
        add(l, r, x, 2 * i + 1);
        add(l, r, x, 2 * i + 2);
        tr[i] = Node(tr[2 * i + 1], tr[2 * i + 2]);
    }
    void get(long long l, long long r, vector<long long> &a, long long i = 0) {
        push(i);
        if (tr[i].r <= l || tr[i].l >= r || tr[i].mi > 0) return;
        if (tr[i].r - tr[i].l == 1) {
            a.push_back(tr[i].l);
            return;
        }
        get(l, r, a, 2 * i + 1);
        get(l, r, a, 2 * i + 2);
    }
    Segment_tree(long long n) {
        tr.resize(4 * n);
        build(0, 0, n);
    }
};

struct MST{
    struct Node{
        long long l, r;
        vector<int> lans;
        Node() {}
        Node (int x, int ind) {
            l = ind;
            r = ind + 1;
            lans.push_back(x);
        }
        Node(Node le, Node ri) {
            l = le.l;
            r = ri.r;
            int x = 0, y = 0;
            while (x < le.lans.size() && y < ri.lans.size()) {
                if (le.lans[x] < ri.lans[y]) {
                    lans.push_back(le.lans[x]);
                    x++;
                } else {
                    lans.push_back(ri.lans[y]);
                    y++;
                }
            }
            while (x < le.lans.size()) {
                lans.push_back(le.lans[x]);
                x++;
            }
            while (y < ri.lans.size()) {
                lans.push_back(ri.lans[y]);
                y++;
            }
        }
    };
    vector<Node> tr;
    void build(long long i, long long l, long long r, vector<int> &a) {
        if (r - l == 1) {
            tr[i] = Node(a[l], l);
            return;
        }
        long long m = (l + r) / 2;
        build(2 * i + 1, l, m, a);
        build(2 * i + 2, m, r, a);
        tr[i] = Node(tr[2 * i + 1], tr[2 * i + 2]);
    }
    int get(long long l, long long r, int x, long long i = 0) {
        if (tr[i].l >= r || tr[i].r <= l) {
            return 0;
        }
        if (l <= tr[i].l && tr[i].r <= r) {
            return (upper_bound(tr[i].lans.begin(), tr[i].lans.end(), x) - tr[i].lans.begin());
        }
        return get(l, r, x, 2 * i + 1) + get(l, r, x, 2 * i + 2);
    }
    MST(vector<int> &a) {
        tr.resize(4 * a.size());
        build(0, 0, a.size(), a);
    }
    MST(){}
};

vector<pair<int, int>> fans;
MST lans;

void init(int N, int Q, const std::vector<int>& a) {
    vector<deque<long long>> v(N + 1);
    for (long long i = 0; i <= N; i++) {
        v[i].push_back(-1);
    }
    Segment_tree tr(N + 1);
    long long le = 0;
    for (long long i = 0; i < N; i++) {
        vector<long long> l;
        tr.add(v[a[i]].back() + 1, i + 1, a[i]);
        tr.add(v[a[i]].front() + 1, i + 1, -1);
        v[a[i]].push_back(i);
        if (v[a[i]].size() > a[i]) {
            le = max(le, (long long)(v[a[i]].front() + 1));
            v[a[i]].pop_front();
        }
        tr.get(le, i + 1, l);
        for (auto &j : l) {
            fans.push_back({j, i + 1});
        }
    }
    sort(fans.begin(), fans.end());
    vector<int> vv(fans.size());
    for (int i = 0; i < fans.size(); i++) {
        vv[i] = fans[i].second;
    }
    lans = MST(vv);
}

long long query(int l, int r) {
    r++;
    int cnt = 0;
    int le = lower_bound(fans.begin(), fans.end(), make_pair(l, 0)) - fans.begin();
    cnt = lans.get(le, fans.size(), r);
    return cnt;
}

int main(){
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr); std::cout.tie(nullptr);
	int n, q; std::cin >> n >> q;
	std::vector<int> input(n);
	for(long long i = 0; i < n; i ++){
		std::cin >> input[i];
	}
	init(n, q, input);
	for(long long i = 0; i < q; i ++){
		long long l, r;
		std::cin >> l >> r;
		std::cout << query(l, r) << " ";
	}
	std::cout << std::endl;
}
