#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>

using namespace std;

const int maxn = 3e5, maxs = 1<<20;
int x[maxn], ft[maxn];

int cn[maxs], mx[maxs], add[maxs];

void push(int v) {
	add[2 * v] += add[v];
	add[2 * v + 1] += add[v];
	mx[2 * v] += add[v];
	mx[2 * v + 1] += add[v];
	add[v] = 0;
}

void build(int v, int l, int r, int x) {
	if (r - l == 1) {
		mx[v] = x;
		cn[v] = 1;
		add[v] = 0;
		return;
	}
	int m = (l + r) / 2;
	build(2 * v, l, m, x);
	build(2 * v + 1, m, r, x);
	mx[v] = x;
	cn[v] = r - l;
	add[v] = 0;
}

void upd(int v, int l, int r, int tl, int tr, int vl) {
	if (l >= tr || tl >= r) return;
	if (l >= tl && r <= tr) {
		mx[v] += vl;
		add[v] += vl;
		return;
	}
	push(v);
	int m = (l + r) / 2;
	upd(2 * v, l, m, tl, tr, vl);
	upd(2 * v + 1, m, r, tl, tr, vl);
	if (mx[2 * v] > mx[2 * v + 1]) {
		mx[v] = mx[2 * v];
		cn[v] = cn[2 * v];
	}
	else if (mx[2 * v] < mx[2 * v + 1]) {
		mx[v] = mx[2 * v + 1];
		cn[v] = cn[2 * v + 1];
	}
	else {
		mx[v] = mx[2 * v];
		cn[v] = cn[2 * v] + cn[2 * v + 1];
	}
}

int get(int v, int l, int r, int tl, int tr) {
	if (l >= tr || tl >= r) return 0;
	if (l >= tl && r <= tr) return (mx[v] == 0) * cn[v];
	int m = (l + r) / 2;
	push(v);
	return get(2 * v, l, m, tl, tr) + get(2 * v + 1, m, r, tl, tr);
}

void init(int N, int Q, const std::vector<int>& a) {
	for (int i = 0; i < N; ++i) {
		x[i] = a[i];
	}
}

long long query(int l, int r) {
	int n = r - l + 1;
	vector<int> v(n), d(n);
	for (int i = l; i <= r; ++i) {
		v[i - l] = x[i];
		d[i - l] = x[i];
	}
	sort(d.begin(), d.end());
	d.resize(unique(d.begin(), d.end()) - d.begin());
	vector<vector<vector<int>>> vc(n + 1);
	for (int i = 0; i < d.size(); ++i) {
		ft[d[i]] = i;
	}
	vector<vector<int>> ids(d.size(), vector<int>(1, -1));
	for (int i = 0; i < n; ++i) {
		ids[ft[v[i]]].push_back(i);
	}
	for (int i = 0; i < d.size(); ++i) {
		ids[i].push_back(n);
		for (int j = 0; j + 1 < ids[i].size(); ++j) {
			vc[ids[i][j] + 1].push_back({ ids[i][j] + 1, ids[i][j + 1], 1 });
			vc[ids[i][j + 1]].push_back({ ids[i][j] + 1, ids[i][j + 1], -1 });
		}
		for (int j = d[i]; j + 1 < ids[i].size(); ++j) {
			vc[ids[i][j]].push_back({ids[i][j - d[i]] + 1, ids[i][j - d[i] + 1] + 1, 1});
			vc[ids[i][j + 1]].push_back({ ids[i][j - d[i]] + 1, ids[i][j - d[i] + 1] + 1, -1 });
		}
	}
	build(1, 0, n,  -((int) d.size()));
	long long rs = 0;
	for (int i = 0; i < n; ++i) {
		for (auto j : vc[i]) {
			upd(1, 0, n, j[0], j[1], j[2]);
		}
		rs += get(1, 0, n, 0, i + 1);
	}
	return rs;
}

/*
int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(nullptr); std::cout.tie(nullptr);
	int n, q; std::cin >> n >> q;
	std::vector<int> input(n);
	for (int i = 0; i < n; i++) {
		std::cin >> input[i];
	}
	init(n, q, input);
	for (int i = 0; i < q; i++) {
		int l, r;
		std::cin >> l >> r;
		std::cout << query(l, r) << "\n";
	}
	std::cout << std::endl;
}
*/