//#include "self_desc.h"

// void init(int N, int Q, const std::vector<int>& a);
// long long query(int l, int r);

#pragma GCC optimize("Os")

#include <vector>
#include <algorithm>
#include <array>
#include <cassert>

//#include <iostream>

namespace __Sol__ {
  using ll = unsigned long long;
  template<class T> using vec = std::vector<T>;

  // problem --> fenwick doesnt work
  // AINT2D dinamic????
  // O(Nlog2N) memorie ---- 1G -- nope
  // aici baieti cred ca se baga MO
  // sqrt(500k) * 800k -- REALLY slow

  // ce penala era asta offline

  // AINT(AIB) -- ai nevoie doar de prefixle la una dintre dimensiuni
  // ca de obicei o belesti ca nu ai memorie!!!!!!

  // cea mai mare structureala din viata mea hands down
  // --- baleere -> tii cate de deasupra ta sunt bune
  // --- range +/-, cu conditia ca - mereu da - la un + de dinainte
  // asta inseamna ca toate lazy-urile sunt pozitive si au semnificatia
  // de dezactivari --> pentru un nod cand dai push cu lazy-ul din parinte
  // lazy(delta, de cate ori ai adaugat inca o linie noua si ai avut min=0 in nod)
  // ar trebuii sa poti sa reconstruiesti totul
  // doamne ajuta
  // macar sa mearga pe subtask-ul offline gen
  // a mers
  // hai cu persistenta dracului

  // nu intra in memorie
  // cel mult O(sqrtN) valori distincte intr-un interval bun


  struct Node {
    unsigned int left:27, right:27; // pentru persistenta
    unsigned int min:19, cntmin:19;
    ll good_up:37;

    constexpr Node():
      left(0), right(0),
      min(0), cntmin(1), good_up(0) {}

    //inline __attribute__((always_inline))
    void add_delta( int __delta ) {
      min += __delta;
    }

    //inline __attribute__((always_inline))
    void add_jumps( int minval, int jumps ) {
      if( min != minval )
        return;

      good_up += jumps * (ll)cntmin;
    }

    void push( Node &left, Node &right ) {
      int delta = min - std::min( left.min, right.min );
      if( delta ){
        left.add_delta( delta );
        right.add_delta( delta );
      }

      ll extra_up = good_up;
      extra_up -= left.good_up;
      extra_up -= right.good_up;

      int good_jumps = int(extra_up / cntmin); // should never go crazy

      if( good_jumps ){
        left.add_jumps( min, good_jumps );
        right.add_jumps( min, good_jumps );
      }
    }

    void pull( const Node &left, const Node &right ) {
      min = std::min( left.min, right.min );
      cntmin = 0;
      if( left.min == min ) cntmin += left.cntmin;
      if( right.min == min ) cntmin += right.cntmin;

      good_up = left.good_up + right.good_up;
    }
  };

  struct AINT {
    std::array<Node, 41666666> aint;
    int n;

    int root;
    int save() const { return root; }

    // pretend like life is good
    void pull( int root ){
      int left = aint[root].left, right = aint[root].right;
      aint[root].pull( aint[left], aint[right] );
    }

    void push( int root ){
      int left = aint[root].left, right = aint[root].right;
      aint[root].push( aint[left], aint[right] );
    }

    int new_node() {
      static int size = 0;
      return size++;
    }

    int copy( int root ) {
      int newroot = new_node();
      aint[newroot] = aint[root];
      return newroot;
    }

    int build( int rl, int rr ) {
      int root = new_node();

      if( rl == rr )
        return root;

      int mij = (rl + rr) >> 1;
      aint[root].left = build( rl, mij );
      aint[root].right = build( mij+1, rr );
      pull( root );

      return root;
    }

    AINT( int n ): n(n), root(build(0, n - 1)) {}

    template<class CB> void walk( int time, int qst, int qdr, CB &&cb ) {
      // printf( "qst = %d, qdr = %d\n", qst, qdr );
      // assert( qst <= qdr );
      // assert( 0 <= qst && qst < n );
      // assert( 0 <= qdr && qdr < n );
      walk( root = copy( time ), 0, n - 1, qst, qdr, cb );
    }

    template<class CB> void walk( int root, int rl, int rr, int qst, int qdr, CB &&cb ) {
      int mij = (rl + rr) >> 1;

      if( qst == rl && qdr == rr ){
        cb( root, rl, rr );
        return;
      }

      int left = aint[root].left = copy( aint[root].left );
      int right = aint[root].right = copy( aint[root].right );
      push( root );

      if( qdr <= mij ){
        walk( left, rl, mij, qst, qdr, cb );
      }else if( qst > mij ){
        walk( right, mij+1, rr, qst, qdr, cb );
      }else{
        walk( left, rl, mij, qst, mij, cb );
        walk( right, mij+1, rr, mij+1, qdr, cb );
      }

      pull( root );
    }

    ll query( int time, int st, int dr ) {
      ll ret = 0;
      walk( time, st, dr, [&]( int root, int rl, int rr ) -> void {
        ret += aint[root].good_up;
      });

      return ret;
    }

    void jump( int st, int dr ) {
      walk( root, st, dr, [&]( int root, int rl, int rr ) -> void {
        aint[root].add_jumps( 0, +1 );
      });
    }

    void add_range( int st, int dr, int delta ) {
      walk( root, st, dr, [&]( int root, int rl, int rr ) -> void {
        aint[root].add_delta( delta );
      });
    }
  };

  struct Rect {
    int x1, x2, y1, y2;
    Rect( int a, int b, int c, int d ): x1(a), x2(b), y1(c), y2(d) {}
  };

  vec<Rect> get_rect( int n, int q, const vec<int> &v ) {
    vec<vec<int>> poz(n + 1);

    for( int val = 1; val <= n; val++ )
      poz[val].push_back( -1 );
    for( int i = 0; i < n; i++ )
      poz[v[i]].push_back( i );
    for( int val = 1; val <= n; val++ )
      poz[val].push_back( n );

    // add +1 to all rectangles that make you angry
    vec<Rect> ret;
    for( int val = 1; val <= n; val++ ){ // because yeah
      if( poz[val].empty() ) continue;

      // firstly, ranges that don't contain you are all good
      // now ranges that contain the right ammount of appearances of you

      if( val >= 2 ){ // [1, val - 1]
        for( int i = 1; i < (int)poz[val].size() - 1; i++ ){
          int last = n;
          if( i + val - 1 < (int)poz[val].size() )
            last = poz[val][i + val - 1];

          ret.emplace_back( poz[val][i - 1] + 1, poz[val][i], poz[val][i], last - 1 );
        }
      }

      { // [val+1, ..]
        for( int i = 1; i + val < (int)poz[val].size() - 1; i++ )
          ret.emplace_back( poz[val][i - 1] + 1, poz[val][i], poz[val][i + val], n - 1 );
      }
    }

    return ret;
  }

  struct Event {
    int x, y1, y2, delta;
  };

  const int MAXN = 3e5;

  vec<int> hist;
  AINT zile(MAXN);
  long long ret;
  void init( int n, int q, const vec<int> &v ) {
    assert( sizeof(Node) == 5 * 4 );
    vec<Rect> rect = get_rect( n, q, v );

    vec<Event> evt;
    for( auto [x1, x2, y1, y2]: rect ){
      evt.push_back({ x1,     y1, y2, +1 });
      evt.push_back({ x2 + 1, y1, y2, -1 });
    }

    std::sort( evt.begin(), evt.end(), []( const Event &a, const Event &b ) -> bool {
      return a.x < b.x;
    } );

    hist.resize(n + 1);

    int eidx = 0;
    for( int time = 0; time <= n; time++ ){
      hist[time] = zile.save();

      while( eidx < (int)evt.size() && evt[eidx].x <= time ){
        auto [_, y1, y2, delta] = evt[eidx];
        zile.add_range( y1, y2, delta );
        eidx++;
      }

      zile.jump( 0, n - 1 );
    }

    ret = -1;
  }

  ll query( int st, int dr ) {
    if( ret == -1 ){
      int len = dr - st + 1;
      ll adjust = (len * (ll)(len - 1) / 2);
      ll qret = (
        + zile.query( hist[dr + 1], st, dr )
        - zile.query( hist[st], st, dr )
      );

      return ret = qret - adjust;
    }

    return ret;
  }
}

void init(int N, int Q, const std::vector<int>& a) { __Sol__::init( N, Q, a ); }
long long query(int l, int r) { return __Sol__::query( l, r ); }

// Lgrader.cpp
#ifndef EVAL
#include<iostream>
#include<vector>

int main(){
  std::ios_base::sync_with_stdio(false);
  std::cin.tie(nullptr); std::cout.tie(nullptr);
  int n, q; std::cin >> n >> q;
  std::vector<int> input(n);
  for(int i = 0; i < n; i ++){
    std::cin >> input[i];
  }
  init(n, q, input);
  for(int i = 0; i < q; i ++){
    int l, r;
    std::cin >> l >> r;
    std::cout << query(l, r) << " ";
  }
  std::cout << std::endl;
}
#endif
