/*
    Zvunq ti alo
    i repetiram dumite pred ogledalo
*/

#include "tunnels.h"
#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <random>
#include <chrono>
#include <vector>
#include <queue>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

template <typename T>
using OSet = __gnu_pbds::tree <T, __gnu_pbds::null_type, std::less <T>, __gnu_pbds::rb_tree_tag, __gnu_pbds::tree_order_statistics_node_update>;

typedef long long llong;
const llong TIME = std::chrono::steady_clock::now().time_since_epoch().count();
const int MAXN = 400 + 10;
const int INF = 1e9;

int n, m;
bool t[MAXN][MAXN];
int prev[MAXN][MAXN];
int next[MAXN][MAXN];
int prefix[MAXN][MAXN];
bool exists[MAXN][MAXN];
int dp[MAXN][MAXN][MAXN];
bool bl[MAXN][MAXN][MAXN];
int choose[MAXN][MAXN][MAXN];
std::vector <int> cells[MAXN];
std::queue <std::pair <int,int>> q;
std::pair <int,int> delta[] = {{-1, 0}, {0, -1}, {0, 1}};
int distA[MAXN][MAXN];
int distB[MAXN][MAXN];

void run(int startX, int startY, int dist[][MAXN])
{
    delta[0].first *= -1;
    for (int i = 1 ; i <= n + 1;  ++i)
    {
        for (int j = 1 ; j <= m + 1 ; ++j)
        {
            dist[i][j] = INF;
        }
    }

    if (startX == n + 1)
    {
        for (int i = 1 ; i <= m ; ++i)
        {
            dist[n + 1][i] = 0;
            q.push({n + 1, i});
        }
    } else
    {
        q.push({startX, startY});
        dist[startX][startY] = 0;
    }

    while (q.size())
    {
        auto [row, col] = q.front();
        q.pop();

        for (auto [dr, dc] : delta)
        {
            if (row + dr >= 1 && row + dr <= n + 1 && col + dc >= 1 && col + dc <= m && !t[row + dr][col + dc] && dist[row + dr][col + dc] == INF)
            {
                dist[row + dr][col + dc] = dist[row][col] + 1;
                q.push({row + dr, col + dc});
            }
        }
    }
}

int rec(int row, int colL, int colR)
{
    // std::cout << "call: " << row << ' ' << colL << ' ' << colR << '\n';
    if (colL == colR)
    {
        bl[row][colL][colR] = true;
        if (row == n) return dp[row][colL][colR] = 0;
        return dp[row][colL][colR] = rec(row + 1, prev[row + 1][cells[row][colL]], next[row + 1][cells[row][colL]]); 
    }

    if (bl[row][colL][colR])
    {
        return dp[row][colL][colR];
    }

    bl[row][colL][colR] = true;
    dp[row][colL][colR] = INF;
    for (int s = colL ; s < colR ; ++s)
    {
        int cost = std::max(rec(row, colL, s), rec(row, s + 1, colR)) + 1;
        if (cost < dp[row][colL][colR])
        {
            dp[row][colL][colR] = cost;
            choose[row][colL][colR] = s;
        }
    }

    // std::cout << "dp: " << row << ' ' << colL << ' ' << colR << " = " << dp[row][colL][colR] << '\n';
    return dp[row][colL][colR];
}

void solve(int N, int M, int k, const std::vector<std::vector<bool>>& blocked)
{
    n = N;
    m = M;
    k++;

    for (int i = 1 ; i <= n ; ++i)
    {
        for (int j = 1 ; j <= m ; ++j)
        {
            t[i][j] = blocked[i - 1][j - 1];
        }
    }

    run(1, k, distA);
    run(n + 1, m, distB);
    if (distB[1][k] == INF)
    {
        return;
    }
    // exit(0);

    for (int i = 1 ; i <= n ; ++i)
    {
        for (int j = 1 ; j <= m ; ++j)
        {
            // std::cout << "here: " << i << ' ' << j << " = " << distA[i][j] << ' ' << distB[i + 1][j] << ' ' << distB[1][k] << '\n';
            if (1 + distA[i][j] + distB[i + 1][j] == distB[1][k]) cells[i].push_back(j);
        }
    }
    
    for (int i = 1 ; i <= n ; ++i)
    {
        {
            int idx = 0;
            int cnt = 0;
            for (int j = 1 ; j <= m ; ++j)
            {
                if (t[i][j]) idx = cnt;
                prev[i][j] = idx;
                cnt += (1 + distA[i][j] + distB[i + 1][j] == distB[1][k]);
            }
        }

        {
            int idx = (int)cells[i].size() - 1;
            int cnt = idx;
            for (int j = m ; j >= 1 ; --j)
            {
                if (t[i][j]) idx = cnt;
                next[i][j] = idx;
                cnt -= (1 + distA[i][j] + distB[i + 1][j] == distB[1][k]);
            }
        }
    }

    rec(1, prev[1][k], next[1][k]);
    int row = 1, colL = prev[1][k], colR = next[1][k], from = k;
    while (row <= n)
    {
        rec(row, colL, colR);
        // std::cout << "call: " << row << ' ' << colL << ' ' << colR << '\n';
        if (colL == colR)
        {
            assert(1 <= cells[row][colL] && cells[row][colL] <= m);
            goDeeper(cells[row][colL] - 1);
            assert(dp[row][colL][colR] != INF);
            if (row == n)
            {
                break;
            }

            from = cells[row][colL];
            colL = prev[row + 1][cells[row][colL]];
            colR = next[row + 1][cells[row][colL]];
            row++;
            continue;
        }

        int s = choose[row][colL][colR];
        // std::cout << "here: " << s << ' ' << from << '\n';
        if (cells[row][s] < from)
        {
            assert(1 <= cells[row][s] && cells[row][s] <= m);
            bool res = investigate(cells[row][s] - 1);
            if (res)
            {
                colR = s;
            } else
            {
                colL = s + 1;
            }
        } else
        {
            assert(1 <= cells[row][s + 1] && cells[row][s + 1] <= m);
            bool res = investigate(cells[row][s + 1] - 1);
            if (!res)
            {
                colR = s;
            } else
            {
                colL = s + 1;
            }
        }
    }
}