/*
    Zvunq ti alo
    i repetiram dumite pred ogledalo
*/

#include <algorithm>
#include <iostream>
#include <numeric>
#include <cassert>
#include <random>
#include <chrono>
#include <vector>

#include <ext/pb_ds/assoc_container.hpp>
#include <ext/pb_ds/tree_policy.hpp>

template <typename T>
using OSet = __gnu_pbds::tree <T, __gnu_pbds::null_type, std::less <T>, __gnu_pbds::rb_tree_tag, __gnu_pbds::tree_order_statistics_node_update>;

typedef long long llong;
const llong TIME = std::chrono::steady_clock::now().time_since_epoch().count();
const int MAXN = 300000 + 10;
const int BUCKET_SIZE = 3;
const int BUCKET_CNT = 600;

int prefix2D[BUCKET_CNT][BUCKET_CNT];
std::vector <int> withEnd[MAXN];
std::vector <int> withBegin[MAXN];

int n;
struct Fenwick
{
    int tree[MAXN];
    void reset()
    {
        std::fill(tree + 1, tree + 1 + n, 0);
    }

    void build(int a[])
    {
        for (int i = 1 ; i <= n ; ++i)
        {
            if (i + (i & (-i)) <= n)
            {
                tree[i + (i & (-i))] += tree[i];
            }
        }
    }

    void update(int idx, int val)
    {
        for (; idx <= n ; idx += idx & (-idx))
        {
            tree[idx] += val;
        }
    }
};

struct LinkedList
{
	int prev[MAXN];
	int next[MAXN];

	void build()
	{
		for (int i = 1 ; i <= n ; ++i)
		{
			prev[i] = i - 1;
			next[i] = i + 1;
		}
	}

	void erase(int idx)
	{
		prev[next[idx]] = prev[idx];
		next[prev[idx]] = next[idx];
	}
};

int last[MAXN];
int prev[MAXN];
std::vector <std::pair <int,int>> ranges;
// std::mt19937 rng(69420);
std::vector <int> indices[MAXN];
LinkedList list;

inline int getBucket(int idx)
{
	return idx / BUCKET_SIZE + 1;
}

inline int beginBucket(int idx)
{
	idx--;
	return std::max(1, idx * BUCKET_SIZE);
}

inline int endBucket(int idx)
{
	idx--;
	return std::min((idx + 1) * BUCKET_SIZE - 1, n);
}

int a[MAXN];
void init(int N, int Q, const std::vector <int> &vals)
{
    n = N;
    for (int i = 1 ; i <= n ; ++i)
    {
        a[i] = vals[i - 1];
    }

    for (int i = 1 ; i <= n ; ++i)
    {
        indices[i].push_back(0);
    }

	for (int i = 1 ; i <= n ; ++i)
	{
		prev[i] = last[a[i]];
		last[a[i]] = i;
	}

	list.build();
	for (int i = 1 ; i <= n ; ++i)
	{
		indices[a[i]].push_back(i);
		if (prev[i] != 0)
		{
			list.erase(prev[i]);
		}

		int bad = 0;
		int from = i;
		int idx = i;

		while (idx > bad && from > bad)
		{
			if (indices[a[idx]].size() - 1 < a[idx])
			{
				break;
			}

			from = std::min(from, indices[a[idx]][indices[a[idx]].size() - a[idx]]);
			bad = std::max(bad, indices[a[idx]][indices[a[idx]].size() - a[idx] - 1]);
			if (list.prev[idx] < from && bad < from)
			{
				withEnd[i].push_back(from);
				withBegin[from].push_back(i);
				prefix2D[getBucket(i)][getBucket(from)]++;
			}

			idx = list.prev[idx];
		}
	}

	for (int i = 1 ; i <= n ; ++i)
	{
		std::sort(withBegin[i].begin(), withBegin[i].end());
	}

	for (int i = 1 ; i <= getBucket(n) ; ++i)
	{
		for (int j = 1 ; j <= getBucket(n) ; ++j)
		{
			prefix2D[i][j] += prefix2D[i - 1][j];
			prefix2D[i][j] += prefix2D[i][j - 1];
			prefix2D[i][j] -= prefix2D[i - 1][j - 1];
		}
	}
}

int sum(int rowB, int colB, int rowE, int colE)
{
	return prefix2D[rowE][colE] - prefix2D[rowB - 1][colE] - prefix2D[rowE][colB - 1] + prefix2D[rowB - 1][colB - 1];
}

llong query(int l, int r)
{
	l++; r++;
	if (getBucket(l) == getBucket(r))
	{
		int res = 0;
		for (int i = l ; i <= r ; ++i)
		{
			int ll = -1, rr = withBegin[i].size(), mid;
			while (ll < rr - 1)
			{
				mid = ll + rr >> 1;
				if (withBegin[i][mid] <= r) ll = mid;
				else rr = mid;
			}

			res += rr;
		}

		return res;
	}

	int end = endBucket(getBucket(l));
	int begin = beginBucket(getBucket(r));
	int res = sum(getBucket(l) + 1, getBucket(l) + 1, getBucket(r) - 1, getBucket(r) - 1);
	// std::cout << "here: " << getBucket(l) << ' ' << (r) << ' ' << end << ' ' << begin << ' ' << res << '\n';
	for (int i = l ; i <= end ; ++i)
	{
		int ll = -1, rr = withBegin[i].size(), mid;
		while (ll < rr - 1)
		{
			mid = ll + rr >> 1;
			if (withBegin[i][mid] <= r) ll = mid;
			else rr = mid;
		}

		res += rr;
	}

	for (int i = begin ; i <= r ; ++i)
	{
		int ll = -1, rr = withEnd[i].size(), mid;
		while (ll < rr - 1)
		{
			mid = ll + rr >> 1;
			if (withEnd[i][mid] > end) ll = mid;
			else rr = mid;
		}

		res += rr;
	}

	return res;
}